var svelteReactiveForm = (function (exports) {
    'use strict';

    function noop() { }
    function assign(tar, src) {
        // @ts-ignore
        for (const k in src)
            tar[k] = src[k];
        return tar;
    }
    function run(fn) {
        return fn();
    }
    function blank_object() {
        return Object.create(null);
    }
    function run_all(fns) {
        fns.forEach(run);
    }
    function is_function(thing) {
        return typeof thing === 'function';
    }
    function safe_not_equal(a, b) {
        return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
    }
    function is_empty(obj) {
        return Object.keys(obj).length === 0;
    }
    function subscribe(store, ...callbacks) {
        if (store == null) {
            return noop;
        }
        const unsub = store.subscribe(...callbacks);
        return unsub.unsubscribe ? () => unsub.unsubscribe() : unsub;
    }
    function get_store_value(store) {
        let value;
        subscribe(store, _ => value = _)();
        return value;
    }
    function create_slot(definition, ctx, $$scope, fn) {
        if (definition) {
            const slot_ctx = get_slot_context(definition, ctx, $$scope, fn);
            return definition[0](slot_ctx);
        }
    }
    function get_slot_context(definition, ctx, $$scope, fn) {
        return definition[1] && fn
            ? assign($$scope.ctx.slice(), definition[1](fn(ctx)))
            : $$scope.ctx;
    }
    function get_slot_changes(definition, $$scope, dirty, fn) {
        if (definition[2] && fn) {
            const lets = definition[2](fn(dirty));
            if ($$scope.dirty === undefined) {
                return lets;
            }
            if (typeof lets === 'object') {
                const merged = [];
                const len = Math.max($$scope.dirty.length, lets.length);
                for (let i = 0; i < len; i += 1) {
                    merged[i] = $$scope.dirty[i] | lets[i];
                }
                return merged;
            }
            return $$scope.dirty | lets;
        }
        return $$scope.dirty;
    }
    function update_slot_base(slot, slot_definition, ctx, $$scope, slot_changes, get_slot_context_fn) {
        if (slot_changes) {
            const slot_context = get_slot_context(slot_definition, ctx, $$scope, get_slot_context_fn);
            slot.p(slot_context, slot_changes);
        }
    }
    function get_all_dirty_from_scope($$scope) {
        if ($$scope.ctx.length > 32) {
            const dirty = [];
            const length = $$scope.ctx.length / 32;
            for (let i = 0; i < length; i++) {
                dirty[i] = -1;
            }
            return dirty;
        }
        return -1;
    }
    function exclude_internal_props(props) {
        const result = {};
        for (const k in props)
            if (k[0] !== '$')
                result[k] = props[k];
        return result;
    }
    function insert(target, node, anchor) {
        target.insertBefore(node, anchor || null);
    }
    function detach(node) {
        if (node.parentNode) {
            node.parentNode.removeChild(node);
        }
    }
    function element(name) {
        return document.createElement(name);
    }
    function text(data) {
        return document.createTextNode(data);
    }
    function space() {
        return text(' ');
    }
    function listen(node, event, handler, options) {
        node.addEventListener(event, handler, options);
        return () => node.removeEventListener(event, handler, options);
    }
    function attr(node, attribute, value) {
        if (value == null)
            node.removeAttribute(attribute);
        else if (node.getAttribute(attribute) !== value)
            node.setAttribute(attribute, value);
    }
    function children(element) {
        return Array.from(element.childNodes);
    }
    function set_style(node, key, value, important) {
        if (value == null) {
            node.style.removeProperty(key);
        }
        else {
            node.style.setProperty(key, value, important ? 'important' : '');
        }
    }

    let current_component;
    function set_current_component(component) {
        current_component = component;
    }
    function get_current_component() {
        if (!current_component)
            throw new Error('Function called outside component initialization');
        return current_component;
    }
    /**
     * Schedules a callback to run immediately before the component is updated after any state change.
     *
     * The first time the callback runs will be before the initial `onMount`
     *
     * https://svelte.dev/docs#run-time-svelte-beforeupdate
     */
    function beforeUpdate(fn) {
        get_current_component().$$.before_update.push(fn);
    }
    /**
     * Schedules a callback to run immediately before the component is unmounted.
     *
     * Out of `onMount`, `beforeUpdate`, `afterUpdate` and `onDestroy`, this is the
     * only one that runs inside a server-side component.
     *
     * https://svelte.dev/docs#run-time-svelte-ondestroy
     */
    function onDestroy(fn) {
        get_current_component().$$.on_destroy.push(fn);
    }
    // TODO figure out if we still want to support
    // shorthand events, or if we want to implement
    // a real bubbling mechanism
    function bubble(component, event) {
        const callbacks = component.$$.callbacks[event.type];
        if (callbacks) {
            // @ts-ignore
            callbacks.slice().forEach(fn => fn.call(this, event));
        }
    }

    const dirty_components = [];
    const binding_callbacks = [];
    let render_callbacks = [];
    const flush_callbacks = [];
    const resolved_promise = /* @__PURE__ */ Promise.resolve();
    let update_scheduled = false;
    function schedule_update() {
        if (!update_scheduled) {
            update_scheduled = true;
            resolved_promise.then(flush);
        }
    }
    function add_render_callback(fn) {
        render_callbacks.push(fn);
    }
    // flush() calls callbacks in this order:
    // 1. All beforeUpdate callbacks, in order: parents before children
    // 2. All bind:this callbacks, in reverse order: children before parents.
    // 3. All afterUpdate callbacks, in order: parents before children. EXCEPT
    //    for afterUpdates called during the initial onMount, which are called in
    //    reverse order: children before parents.
    // Since callbacks might update component values, which could trigger another
    // call to flush(), the following steps guard against this:
    // 1. During beforeUpdate, any updated components will be added to the
    //    dirty_components array and will cause a reentrant call to flush(). Because
    //    the flush index is kept outside the function, the reentrant call will pick
    //    up where the earlier call left off and go through all dirty components. The
    //    current_component value is saved and restored so that the reentrant call will
    //    not interfere with the "parent" flush() call.
    // 2. bind:this callbacks cannot trigger new flush() calls.
    // 3. During afterUpdate, any updated components will NOT have their afterUpdate
    //    callback called a second time; the seen_callbacks set, outside the flush()
    //    function, guarantees this behavior.
    const seen_callbacks = new Set();
    let flushidx = 0; // Do *not* move this inside the flush() function
    function flush() {
        // Do not reenter flush while dirty components are updated, as this can
        // result in an infinite loop. Instead, let the inner flush handle it.
        // Reentrancy is ok afterwards for bindings etc.
        if (flushidx !== 0) {
            return;
        }
        const saved_component = current_component;
        do {
            // first, call beforeUpdate functions
            // and update components
            try {
                while (flushidx < dirty_components.length) {
                    const component = dirty_components[flushidx];
                    flushidx++;
                    set_current_component(component);
                    update(component.$$);
                }
            }
            catch (e) {
                // reset dirty state to not end up in a deadlocked state and then rethrow
                dirty_components.length = 0;
                flushidx = 0;
                throw e;
            }
            set_current_component(null);
            dirty_components.length = 0;
            flushidx = 0;
            while (binding_callbacks.length)
                binding_callbacks.pop()();
            // then, once components are updated, call
            // afterUpdate functions. This may cause
            // subsequent updates...
            for (let i = 0; i < render_callbacks.length; i += 1) {
                const callback = render_callbacks[i];
                if (!seen_callbacks.has(callback)) {
                    // ...so guard against infinite loops
                    seen_callbacks.add(callback);
                    callback();
                }
            }
            render_callbacks.length = 0;
        } while (dirty_components.length);
        while (flush_callbacks.length) {
            flush_callbacks.pop()();
        }
        update_scheduled = false;
        seen_callbacks.clear();
        set_current_component(saved_component);
    }
    function update($$) {
        if ($$.fragment !== null) {
            $$.update();
            run_all($$.before_update);
            const dirty = $$.dirty;
            $$.dirty = [-1];
            $$.fragment && $$.fragment.p($$.ctx, dirty);
            $$.after_update.forEach(add_render_callback);
        }
    }
    /**
     * Useful for example to execute remaining `afterUpdate` callbacks before executing `destroy`.
     */
    function flush_render_callbacks(fns) {
        const filtered = [];
        const targets = [];
        render_callbacks.forEach((c) => fns.indexOf(c) === -1 ? filtered.push(c) : targets.push(c));
        targets.forEach((c) => c());
        render_callbacks = filtered;
    }
    const outroing = new Set();
    let outros;
    function transition_in(block, local) {
        if (block && block.i) {
            outroing.delete(block);
            block.i(local);
        }
    }
    function transition_out(block, local, detach, callback) {
        if (block && block.o) {
            if (outroing.has(block))
                return;
            outroing.add(block);
            outros.c.push(() => {
                outroing.delete(block);
                if (callback) {
                    if (detach)
                        block.d(1);
                    callback();
                }
            });
            block.o(local);
        }
        else if (callback) {
            callback();
        }
    }
    function mount_component(component, target, anchor, customElement) {
        const { fragment, after_update } = component.$$;
        fragment && fragment.m(target, anchor);
        if (!customElement) {
            // onMount happens before the initial afterUpdate
            add_render_callback(() => {
                const new_on_destroy = component.$$.on_mount.map(run).filter(is_function);
                // if the component was destroyed immediately
                // it will update the `$$.on_destroy` reference to `null`.
                // the destructured on_destroy may still reference to the old array
                if (component.$$.on_destroy) {
                    component.$$.on_destroy.push(...new_on_destroy);
                }
                else {
                    // Edge case - component was destroyed immediately,
                    // most likely as a result of a binding initialising
                    run_all(new_on_destroy);
                }
                component.$$.on_mount = [];
            });
        }
        after_update.forEach(add_render_callback);
    }
    function destroy_component(component, detaching) {
        const $$ = component.$$;
        if ($$.fragment !== null) {
            flush_render_callbacks($$.after_update);
            run_all($$.on_destroy);
            $$.fragment && $$.fragment.d(detaching);
            // TODO null out other refs, including component.$$ (but need to
            // preserve final state?)
            $$.on_destroy = $$.fragment = null;
            $$.ctx = [];
        }
    }
    function make_dirty(component, i) {
        if (component.$$.dirty[0] === -1) {
            dirty_components.push(component);
            schedule_update();
            component.$$.dirty.fill(0);
        }
        component.$$.dirty[(i / 31) | 0] |= (1 << (i % 31));
    }
    function init(component, options, instance, create_fragment, not_equal, props, append_styles, dirty = [-1]) {
        const parent_component = current_component;
        set_current_component(component);
        const $$ = component.$$ = {
            fragment: null,
            ctx: [],
            // state
            props,
            update: noop,
            not_equal,
            bound: blank_object(),
            // lifecycle
            on_mount: [],
            on_destroy: [],
            on_disconnect: [],
            before_update: [],
            after_update: [],
            context: new Map(options.context || (parent_component ? parent_component.$$.context : [])),
            // everything else
            callbacks: blank_object(),
            dirty,
            skip_bound: false,
            root: options.target || parent_component.$$.root
        };
        append_styles && append_styles($$.root);
        let ready = false;
        $$.ctx = instance
            ? instance(component, options.props || {}, (i, ret, ...rest) => {
                const value = rest.length ? rest[0] : ret;
                if ($$.ctx && not_equal($$.ctx[i], $$.ctx[i] = value)) {
                    if (!$$.skip_bound && $$.bound[i])
                        $$.bound[i](value);
                    if (ready)
                        make_dirty(component, i);
                }
                return ret;
            })
            : [];
        $$.update();
        ready = true;
        run_all($$.before_update);
        // `false` as a special case of no DOM component
        $$.fragment = create_fragment ? create_fragment($$.ctx) : false;
        if (options.target) {
            if (options.hydrate) {
                const nodes = children(options.target);
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.l(nodes);
                nodes.forEach(detach);
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.c();
            }
            if (options.intro)
                transition_in(component.$$.fragment);
            mount_component(component, options.target, options.anchor, options.customElement);
            flush();
        }
        set_current_component(parent_component);
    }
    /**
     * Base class for Svelte components. Used when dev=false.
     */
    class SvelteComponent {
        $destroy() {
            destroy_component(this, 1);
            this.$destroy = noop;
        }
        $on(type, callback) {
            if (!is_function(callback)) {
                return noop;
            }
            const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
            callbacks.push(callback);
            return () => {
                const index = callbacks.indexOf(callback);
                if (index !== -1)
                    callbacks.splice(index, 1);
            };
        }
        $set($$props) {
            if (this.$$set && !is_empty($$props)) {
                this.$$.skip_bound = true;
                this.$$set($$props);
                this.$$.skip_bound = false;
            }
        }
    }

    const subscriber_queue = [];
    /**
     * Creates a `Readable` store that allows reading by subscription.
     * @param value initial value
     * @param {StartStopNotifier} [start]
     */
    function readable(value, start) {
        return {
            subscribe: writable(value, start).subscribe
        };
    }
    /**
     * Create a `Writable` store that allows both updating and reading by subscription.
     * @param {*=}value initial value
     * @param {StartStopNotifier=} start
     */
    function writable(value, start = noop) {
        let stop;
        const subscribers = new Set();
        function set(new_value) {
            if (safe_not_equal(value, new_value)) {
                value = new_value;
                if (stop) { // store is ready
                    const run_queue = !subscriber_queue.length;
                    for (const subscriber of subscribers) {
                        subscriber[1]();
                        subscriber_queue.push(subscriber, value);
                    }
                    if (run_queue) {
                        for (let i = 0; i < subscriber_queue.length; i += 2) {
                            subscriber_queue[i][0](subscriber_queue[i + 1]);
                        }
                        subscriber_queue.length = 0;
                    }
                }
            }
        }
        function update(fn) {
            set(fn(value));
        }
        function subscribe(run, invalidate = noop) {
            const subscriber = [run, invalidate];
            subscribers.add(subscriber);
            if (subscribers.size === 1) {
                stop = start(set) || noop;
            }
            run(value);
            return () => {
                subscribers.delete(subscriber);
                if (subscribers.size === 0 && stop) {
                    stop();
                    stop = null;
                }
            };
        }
        return { set, update, subscribe };
    }

    /* packages/svelte-reactive-form/src/components/Field.svelte generated by Svelte v3.59.2 */

    const get_default_slot_changes = dirty => ({
    	pending: dirty & /*$state$*/ 8,
    	valid: dirty & /*$state$*/ 8,
    	errors: dirty & /*$state$*/ 8,
    	dirty: dirty & /*$state$*/ 8,
    	touched: dirty & /*$state$*/ 8,
    	value: dirty & /*$state$*/ 8
    });

    const get_default_slot_context = ctx => ({
    	pending: /*$state$*/ ctx[3].pending,
    	valid: /*$state$*/ ctx[3].valid,
    	errors: /*$state$*/ ctx[3].errors,
    	dirty: /*$state$*/ ctx[3].dirty,
    	touched: /*$state$*/ ctx[3].touched,
    	value: /*$state$*/ ctx[3].value,
    	onChange: /*onChange*/ ctx[4],
    	onFocus: /*onFocus*/ ctx[5],
    	onBlur: /*onBlur*/ ctx[6]
    });

    // (57:0) {#if type != "none"}
    function create_if_block(ctx) {
    	let input;
    	let input_value_value;
    	let mounted;
    	let dispose;

    	return {
    		c() {
    			input = element("input");
    			attr(input, "name", /*name*/ ctx[0]);
    			attr(input, "type", /*type*/ ctx[1]);
    			input.value = input_value_value = /*$state$*/ ctx[3].value;
    			set_style(input, "display", "none");
    		},
    		m(target, anchor) {
    			insert(target, input, anchor);

    			if (!mounted) {
    				dispose = listen(input, "input", /*input_handler*/ ctx[15]);
    				mounted = true;
    			}
    		},
    		p(ctx, dirty) {
    			if (dirty & /*name*/ 1) {
    				attr(input, "name", /*name*/ ctx[0]);
    			}

    			if (dirty & /*type*/ 2) {
    				attr(input, "type", /*type*/ ctx[1]);
    			}

    			if (dirty & /*$state$*/ 8 && input_value_value !== (input_value_value = /*$state$*/ ctx[3].value) && input.value !== input_value_value) {
    				input.value = input_value_value;
    			}
    		},
    		d(detaching) {
    			if (detaching) detach(input);
    			mounted = false;
    			dispose();
    		}
    	};
    }

    function create_fragment(ctx) {
    	let t;
    	let current;
    	let if_block = /*type*/ ctx[1] != "none" && create_if_block(ctx);
    	const default_slot_template = /*#slots*/ ctx[14].default;
    	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[13], get_default_slot_context);

    	return {
    		c() {
    			if (if_block) if_block.c();
    			t = space();
    			if (default_slot) default_slot.c();
    		},
    		m(target, anchor) {
    			if (if_block) if_block.m(target, anchor);
    			insert(target, t, anchor);

    			if (default_slot) {
    				default_slot.m(target, anchor);
    			}

    			current = true;
    		},
    		p(ctx, [dirty]) {
    			if (/*type*/ ctx[1] != "none") {
    				if (if_block) {
    					if_block.p(ctx, dirty);
    				} else {
    					if_block = create_if_block(ctx);
    					if_block.c();
    					if_block.m(t.parentNode, t);
    				}
    			} else if (if_block) {
    				if_block.d(1);
    				if_block = null;
    			}

    			if (default_slot) {
    				if (default_slot.p && (!current || dirty & /*$$scope, $state$*/ 8200)) {
    					update_slot_base(
    						default_slot,
    						default_slot_template,
    						ctx,
    						/*$$scope*/ ctx[13],
    						!current
    						? get_all_dirty_from_scope(/*$$scope*/ ctx[13])
    						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[13], dirty, get_default_slot_changes),
    						get_default_slot_context
    					);
    				}
    			}
    		},
    		i(local) {
    			if (current) return;
    			transition_in(default_slot, local);
    			current = true;
    		},
    		o(local) {
    			transition_out(default_slot, local);
    			current = false;
    		},
    		d(detaching) {
    			if (if_block) if_block.d(detaching);
    			if (detaching) detach(t);
    			if (default_slot) default_slot.d(detaching);
    		}
    	};
    }

    function instance($$self, $$props, $$invalidate) {
    	let $state$,
    		$$unsubscribe_state$ = noop,
    		$$subscribe_state$ = () => ($$unsubscribe_state$(), $$unsubscribe_state$ = subscribe(state$, $$value => $$invalidate(3, $state$ = $$value)), state$);

    	$$self.$$.on_destroy.push(() => $$unsubscribe_state$());
    	let { $$slots: slots = {}, $$scope } = $$props;
    	let { name = "" } = $$props;
    	let { defaultValue = undefined } = $$props;
    	let { bail = false } = $$props;
    	let { validateOnBlur = false } = $$props;
    	let { validateOnMount = false } = $$props;
    	let { control } = $$props;
    	let { rules = "" } = $$props;
    	let { type = "none" } = $$props;
    	const form = get_store_value(control);
    	const { register, unregister, setValue, setTouched } = form;

    	// reactive state
    	let state$ = register(name, {
    		defaultValue,
    		rules,
    		validateOnBlur,
    		validateOnMount,
    		bail
    	});

    	$$subscribe_state$();
    	let cache = Object.assign({}, $$props);

    	beforeUpdate(() => {
    		if (JSON.stringify(rules) !== JSON.stringify(cache.rules)) {
    			const value = $state$.value;
    			cache = Object.assign({}, cache, { defaultValue: value, value, rules });
    			$$subscribe_state$($$invalidate(2, state$ = register(name, cache)));
    		}
    	});

    	const onChange = e => {
    		let value = e;

    		if (e.target) {
    			const target = e.target;
    			value = target.value;
    		} else if (e.currentTarget) {
    			const target = e.currentTarget;
    			value = target.value;
    		} else if (e instanceof CustomEvent) {
    			value = e.detail;
    		}

    		setValue(name, value);
    	};

    	const onFocus = () => {
    		setTouched(name, true);
    	};

    	const onBlur = () => {
    		setTouched(name, false);
    	};

    	onDestroy(() => {
    		unregister(name);
    	});

    	function input_handler(event) {
    		bubble.call(this, $$self, event);
    	}

    	$$self.$$set = $$new_props => {
    		$$invalidate(22, $$props = assign(assign({}, $$props), exclude_internal_props($$new_props)));
    		if ('name' in $$new_props) $$invalidate(0, name = $$new_props.name);
    		if ('defaultValue' in $$new_props) $$invalidate(7, defaultValue = $$new_props.defaultValue);
    		if ('bail' in $$new_props) $$invalidate(8, bail = $$new_props.bail);
    		if ('validateOnBlur' in $$new_props) $$invalidate(9, validateOnBlur = $$new_props.validateOnBlur);
    		if ('validateOnMount' in $$new_props) $$invalidate(10, validateOnMount = $$new_props.validateOnMount);
    		if ('control' in $$new_props) $$invalidate(11, control = $$new_props.control);
    		if ('rules' in $$new_props) $$invalidate(12, rules = $$new_props.rules);
    		if ('type' in $$new_props) $$invalidate(1, type = $$new_props.type);
    		if ('$$scope' in $$new_props) $$invalidate(13, $$scope = $$new_props.$$scope);
    	};

    	$$props = exclude_internal_props($$props);

    	return [
    		name,
    		type,
    		state$,
    		$state$,
    		onChange,
    		onFocus,
    		onBlur,
    		defaultValue,
    		bail,
    		validateOnBlur,
    		validateOnMount,
    		control,
    		rules,
    		$$scope,
    		slots,
    		input_handler
    	];
    }

    class Field extends SvelteComponent {
    	constructor(options) {
    		super();

    		init(this, options, instance, create_fragment, safe_not_equal, {
    			name: 0,
    			defaultValue: 7,
    			bail: 8,
    			validateOnBlur: 9,
    			validateOnMount: 10,
    			control: 11,
    			rules: 12,
    			type: 1
    		});
    	}
    }

    const RULES = {};
    const defineRule = (ruleName, cb) => {
        RULES[ruleName] = cb;
    };
    const resolveRule = (ruleName) => {
        const cb = RULES[ruleName];
        return cb;
    };

    /**
     * Transform the callback to a promise
     *
     * @param fn function can be async or not async
     * @returns Async function
     */
    const toPromise = (fn) => {
        return function (...args) {
            const value = fn.apply(null, args);
            if (value && typeof value.then === "function") {
                return value;
            }
            if (typeof value === "function") {
                return Promise.resolve(value.apply(null, args));
            }
            return Promise.resolve(value);
        };
    };
    /**
     * Patch an object with the nested key value
     *
     * @param data the source object
     * @param key the object key we want to append
     * @param value the value of the object's key
     * @returns new source object
     *
     * ### Example
     * ```js
     * normalizeObject({ z: 440.056 }, "a[0].b.c[2]", "hello world!")
     * ```
     */
    const normalizeObject = (src, key, value) => {
        const data = Object.assign({}, src);
        if (!key)
            return data;
        const escape = key.match(/^\[(.*)\]$/);
        // split the key by dot
        const queue = [
            [data, escape ? [escape[1]] : key.split(".")]
        ];
        while (queue.length > 0) {
            const first = queue.shift();
            const paths = first[1];
            const name = paths.shift();
            const result = name.match(/^([a-z\_\.]+)((\[\d+\])+)/i);
            if (result && result.length > 2) {
                const name = result[1];
                // if it's not array, assign it and make it as array
                if (!Array.isArray(first[0][name])) {
                    first[0][name] = [];
                }
                let cur = first[0][name];
                // get array indexes from the key
                const indexs = result[2].replace(/^\[+|\]+$/g, "").split("][");
                while (indexs.length > 0) {
                    // convert the the index string to number
                    const index = parseInt(indexs.shift(), 10);
                    // if nested index is last position && parent is last position
                    if (indexs.length === 0) {
                        if (paths.length === 0) {
                            cur[index] = value;
                        }
                        else {
                            if (!cur[index]) {
                                cur[index] = {};
                            }
                        }
                    }
                    else if (!cur[index]) {
                        // set to empty array if it's undefined
                        cur[index] = [];
                    }
                    cur = cur[index];
                }
                if (paths.length > 0) {
                    queue.push([cur, paths]);
                }
                continue;
            }
            if (paths.length === 0) {
                first[0][name] = value;
            }
            else {
                if (!first[0][name]) {
                    first[0][name] = {};
                }
                queue.push([first[0][name], paths]);
            }
        }
        return data;
    };

    const defaultFormState = {
        dirty: false,
        submitting: false,
        touched: false,
        pending: false,
        valid: false
    };
    const defaultFieldState = {
        defaultValue: "",
        value: "",
        pending: false,
        dirty: false,
        touched: false,
        valid: false,
        errors: []
    };
    const INPUT_FIELDS = ["INPUT", "SELECT", "TEXTAREA"];
    const _strToValidator = (rule) => {
        const params = rule.split(/:/g);
        const name = params.shift();
        return {
            name,
            validate: toPromise(resolveRule(name)),
            params: params[0] ? params[0].split(",").map((v) => decodeURIComponent(v)) : []
        };
    };
    const useForm = (config = { validateOnChange: true }) => {
        // cache for form fields
        const cache = new Map();
        // global state for form
        const form$ = writable(Object.assign({}, defaultFormState));
        const anyNonDirty = new Map();
        const anyPending = new Map();
        const anyNonTouched = new Map();
        const anyInvalid = new Map();
        // errors should be private variable
        const errors$ = writable({});
        const _updateForm = () => {
            form$.update((v) => Object.assign(v, {
                valid: anyInvalid.size === 0,
                dirty: anyNonDirty.size === 0,
                touched: anyNonTouched.size === 0,
                pending: anyPending.size > 0
            }));
        };
        const _useLocalStore = (path, state) => {
            const { subscribe, set, update } = writable(Object.assign({}, defaultFieldState, state));
            let unsubscribe;
            const unsubscribeStore = () => {
                unsubscribe && unsubscribe();
                unsubscribe = null;
            };
            return {
                set,
                update,
                destroy() {
                    cache.delete(path); // clean our cache
                    anyInvalid.delete(path);
                    anyNonDirty.delete(path);
                    anyNonTouched.delete(path);
                    anyPending.delete(path);
                    _updateForm();
                    unsubscribeStore();
                },
                subscribe(run, invalidate) {
                    unsubscribe = subscribe(run, invalidate);
                    return unsubscribeStore;
                }
            };
        };
        const _setStore = (path, state = {}) => {
            const store$ = _useLocalStore(path, state);
            cache.set(path, [store$, [], { bail: false }]);
        };
        const register = (path, option = {}) => {
            const value = option.defaultValue;
            const isNotEmpty = value !== undefined && value !== null;
            const store$ = _useLocalStore(path, { value: value, dirty: isNotEmpty });
            let ruleExprs = [];
            const { bail = false, rules = [] } = option;
            const typeOfRule = typeof rules;
            if (typeOfRule === "string") {
                ruleExprs = (rules.match(/[^\|]+/g) || []).map((v) => _strToValidator(v));
            }
            else if (Array.isArray(rules)) {
                ruleExprs = rules.reduce((acc, rule) => {
                    const typeOfVal = typeof rule;
                    if (!rule)
                        return acc; // skip null, undefined etc
                    if (typeOfVal === "string") {
                        rule = rule.trim();
                        rule && acc.push(_strToValidator(rule));
                    }
                    else if (typeOfVal === "function") {
                        rule = rule;
                        if (rule.name === "")
                            ;
                        acc.push({
                            name: rule.name,
                            validate: toPromise(rule),
                            params: []
                        });
                    }
                    return acc;
                }, []);
            }
            else if (typeOfRule !== null && typeOfRule === "object") {
                ruleExprs = Object.entries(rules).reduce((acc, cur) => {
                    const [name, params] = cur;
                    acc.push({
                        name,
                        validate: toPromise(resolveRule(name)),
                        params: Array.isArray(params) ? params : [params]
                    });
                    return acc;
                }, []);
            }
            else ;
            const field = [store$, ruleExprs, { bail }];
            cache.set(path, field);
            if (isNotEmpty && option.validateOnMount) {
                _validate(field, path, {});
            }
            if (config.validateOnChange) {
                // on every state change, it will update the form
                store$.subscribe((state) => {
                    if (state.valid)
                        anyInvalid.delete(path);
                    else if (!state.valid)
                        anyInvalid.set(path, true);
                    if (state.dirty)
                        anyNonDirty.delete(path);
                    else if (!state.dirty)
                        anyNonDirty.set(path, true);
                    if (!state.pending)
                        anyPending.delete(path);
                    else if (state.pending)
                        anyPending.set(path, true);
                    if (state.touched)
                        anyNonTouched.delete(path);
                    else if (!state.touched)
                        anyNonTouched.set(path, true);
                    _updateForm();
                });
            }
            return {
                subscribe: store$.subscribe
            };
        };
        const unregister = (path) => {
            if (cache.has(path)) {
                // clear subscriptions and cache
                cache.get(path)[0].destroy();
            }
        };
        const setValue = (e, value) => {
            let path = e;
            if (e instanceof Event) {
                const target = e.target;
                path = target.name || target.id;
                value = target.value;
            }
            if (cache.has(path)) {
                const field = cache.get(path);
                if (config.validateOnChange) {
                    _validate(field, path, { dirty: true, value });
                }
                else {
                    field[0].update((v) => Object.assign(v, { dirty: true, value }));
                }
            }
            else {
                _setStore(path);
            }
        };
        const setError = (path, errors) => {
            if (cache.has(path)) {
                const [store$] = cache.get(path);
                store$.update((v) => Object.assign(v, { errors }));
            }
            else {
                _setStore(path, { errors });
            }
        };
        const setTouched = (path, touched) => {
            if (cache.has(path)) {
                const field = cache.get(path);
                _validate(field, path, { touched });
            }
        };
        const getValue = (path) => {
            if (cache.has(path)) {
                const [store$] = cache.get(path);
                const state = get_store_value(store$);
                return state.value;
            }
            return null;
        };
        const _useField = (node, option = {}) => {
            option = Object.assign({ rules: [], defaultValue: "" }, option);
            const selector = INPUT_FIELDS.join(",");
            while (!INPUT_FIELDS.includes(node.nodeName)) {
                const el = node.querySelector(selector);
                node = el;
                if (el)
                    break;
            }
            const name = node.name || node.id;
            const { defaultValue, rules } = option;
            const state$ = register(name, { defaultValue, rules });
            const onChange = (e) => {
                setValue(name, e.currentTarget.value);
            };
            const onBlur = (_) => {
                setTouched(name, true);
            };
            if (defaultValue) {
                node.setAttribute("value", defaultValue);
            }
            const listeners = [];
            const _attachEvent = (event, cb, opts) => {
                node.addEventListener(event, cb, opts);
                listeners.push([event, cb]);
            };
            const _detachEvents = () => {
                for (let i = 0, len = listeners.length; i < len; i++) {
                    const [event, cb] = listeners[i];
                    node.removeEventListener(event, cb);
                }
            };
            _attachEvent("blur", onBlur, { once: true });
            if (config.validateOnChange) {
                _attachEvent("input", onChange);
                _attachEvent("change", onChange);
            }
            if (option.validateOnMount) {
                const field = cache.get(name);
                _validate(field, name, { value: defaultValue });
            }
            let unsubscribe;
            if (option.handleChange) {
                unsubscribe = state$.subscribe((v) => {
                    option.handleChange(v, node);
                });
            }
            return {
                destroy() {
                    _detachEvents();
                    unregister(name);
                    unsubscribe && unsubscribe();
                }
            };
        };
        const reset = (values, option) => {
            const defaultOption = {
                dirtyFields: false,
                errors: false
            };
            option = Object.assign(defaultOption, option || {});
            if (option.errors) {
                errors$.set({}); // reset errors
            }
            const fields = Array.from(cache.values());
            for (let i = 0, len = fields.length; i < len; i++) {
                const [store$] = fields[i];
                store$.update((v) => {
                    const { defaultValue } = v;
                    return Object.assign({}, defaultFieldState, {
                        defaultValue,
                        value: defaultValue
                    });
                });
            }
        };
        const useField = (node, option = {}) => {
            let field = _useField(node, option);
            return {
                update(newOption = {}) {
                    field.destroy(); // reset
                    field = _useField(node, newOption);
                },
                destroy() {
                    field.destroy();
                }
            };
        };
        const onSubmit = (successCallback, errorCallback) => async (e) => {
            form$.update((v) => Object.assign(v, { submitting: true }));
            e.preventDefault();
            e.stopPropagation();
            errors$.set({}); // reset errors
            let data = {}, valid = true;
            const { elements = [] } = e.currentTarget;
            for (let i = 0, len = elements.length; i < len; i++) {
                const el = elements[i];
                const name = el.name || el.id;
                let value = el.value || "";
                if (!name)
                    continue;
                if (config.resolver) {
                    data = normalizeObject(data, name, value);
                    continue;
                }
                // TODO: shouldn't only loop elements, should check cache keys which not exists in elements as well
                if (cache.has(name)) {
                    const field = cache.get(name);
                    // TODO: check checkbox and radio
                    const { nodeName, type } = el;
                    switch (type) {
                        case "checkbox":
                            value = el.checked ? value : "";
                            break;
                    }
                    const result = await _validate(field, name, { value });
                    valid = valid && result.valid; // update valid
                    data = normalizeObject(data, name, value);
                }
            }
            if (config.resolver) {
                try {
                    await config.resolver.validate(data);
                }
                catch (e) {
                    valid = false;
                    errors$.set(e);
                }
            }
            if (valid) {
                await toPromise(successCallback)(data, e);
            }
            else {
                errorCallback && errorCallback(get_store_value(errors$), e);
            }
            // submitting should end only after execute user callback
            form$.update((v) => Object.assign(v, { valid, submitting: false }));
        };
        const _validate = ([store$, validators, { bail }], path, newState) => {
            const promises = [];
            let state = get_store_value(store$);
            state = Object.assign(state, newState);
            const { value } = state;
            if (validators.length === 0) {
                state = Object.assign(state, { errors: [], valid: true });
                store$.set(state);
                return Promise.resolve(state);
            }
            form$.update((v) => Object.assign(v, { pending: true, valid: false }));
            store$.set(Object.assign(state, {
                // dirty: v.dirty ? true : soft ? false : true,
                errors: [],
                pending: true,
                value
            }));
            const len = validators.length;
            let i = 0;
            for (i = 0; i < len; i++) {
                const { validate, params } = validators[i];
                promises.push(validate(value, params, context));
            }
            if (bail) {
                return new Promise(async (resolve) => {
                    for (i = 0; i < len; i++) {
                        const result = await promises[i];
                        if (typeof result === "string") {
                            store$.update((v) => Object.assign(v, {
                                pending: false,
                                errors: [result],
                                valid: false
                            }));
                            resolve(get_store_value(store$));
                            return;
                        }
                    }
                    store$.update((v) => Object.assign(v, {
                        pending: false,
                        errors: [],
                        valid: true
                    }));
                    resolve(get_store_value(store$));
                });
            }
            return Promise.all(promises).then((result) => {
                const errors = result.filter((v) => v !== true);
                const valid = errors.length === 0;
                store$.update((v) => Object.assign(v, { pending: false, errors, valid }));
                return Promise.resolve(get_store_value(store$));
            });
        };
        const validate = (paths = Array.from(cache.keys())) => {
            if (!Array.isArray(paths))
                paths = [paths];
            const promises = [];
            let data = {};
            for (let i = 0, len = paths.length; i < len; i++) {
                if (!cache.has(paths[i]))
                    continue;
                const field = cache.get(paths[i]);
                const state = get_store_value(field[0]);
                promises.push(_validate(field, paths[i], state.value));
                data = normalizeObject(data, paths[i], state.value);
            }
            return Promise.all(promises).then((result) => {
                return {
                    valid: result.every((state) => state.valid),
                    data: data
                };
            });
        };
        const getValues = () => {
            let data = {};
            for (const [name, [store$]] of cache.entries()) {
                const state = get_store_value(store$);
                data = normalizeObject(data, name, state.value);
            }
            return data;
        };
        const context = {
            register,
            unregister,
            setValue,
            getValue,
            setError,
            setTouched,
            getValues,
            reset
        };
        return {
            control: readable(context, () => { }),
            subscribe(run, invalidate) {
                const unsubscribe = form$.subscribe(run, invalidate);
                return () => {
                    // prevent memory leak
                    unsubscribe();
                    cache.clear(); // clean our cache
                };
            },
            errors: {
                subscribe: errors$.subscribe
            },
            field: useField,
            register,
            unregister,
            setValue,
            getValue,
            setError,
            setTouched,
            getValues,
            onSubmit,
            reset,
            validate
        };
    };

    exports.Field = Field;
    exports.defineRule = defineRule;
    exports.useForm = useForm;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({});
